@@ -1,4 +1,6 @@
 #![cfg_attr(docsrs, feature(doc_cfg))]
+#![warn(clippy::pedantic)]
+#![forbid(unsafe_code)]
 //! Syntax aware debug printing.
 //!
 //! Makes use of `syn` and `prettyplease` in order to provide the most
@@ -7,7 +9,7 @@
 //! # Example usage
 //!
 //! ```
-//! use dbg_pls::{debug, DebugPls};
+//! use dbg_pls::{pretty, DebugPls};
 //!
 //! #[derive(DebugPls, Copy, Clone)]
 //! pub struct Demo {
@@ -18,7 +20,7 @@
 //! let mut val = [Demo { foo: 5, bar: "hello" }; 10];
 //! val[6].bar = "Hello, world! I am a very long string";
 //!
-//! let output = format!("{}", debug(&val));
+//! let output = format!("{}", pretty(&val));
 //! let expected = r#"[
 //!     Demo { foo: 5, bar: "hello" },
 //!     Demo { foo: 5, bar: "hello" },
@@ -57,6 +59,12 @@
 //! Outputs:
 //!
 //! ![](https://raw.githubusercontent.com/conradludgate/dbg-pls/5dee03187a3f83693739e0288d56da5980e1d486/readme/highlighted.png)
+//!
+//! # How it works
+//!
+//! All [`DebugPls`] implementations are forced to output only valid
+//! [`syn::Expr`] values. These are then formatted using [`prettyplease::unparse`].
+//! Finally,
 
 use syn::__private::{Span, TokenStream2};
 
@@ -78,7 +86,7 @@ pub use debug_tuple_struct::DebugTupleStruct;
 #[cfg(feature = "pretty")]
 mod pretty;
 #[cfg(feature = "pretty")]
-pub use pretty::debug;
+pub use pretty::pretty;
 
 #[cfg(feature = "colors")]
 mod colors;
@@ -89,6 +97,14 @@ pub use colors::color;
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 pub use dbg_pls_derive::DebugPls;
 
+#[doc(hidden)]
+pub mod __private {
+    #[cfg(feature = "pretty")]
+    pub use crate::pretty::Str as PrettyStr;
+    #[cfg(feature = "colors")]
+    pub use crate::colors::ColorStr;
+}
+
 /// Syntax aware pretty-printed debug formatting.
 ///
 /// `DebugPls` should format the output in a programmer-facing, debugging context.
@@ -100,7 +116,7 @@ pub use dbg_pls_derive::DebugPls;
 /// Deriving an implementation:
 ///
 /// ```
-/// use dbg_pls::{debug, DebugPls};
+/// use dbg_pls::{pretty, DebugPls};
 /// #[derive(DebugPls)]
 /// struct Point {
 ///     x: i32,
@@ -109,13 +125,13 @@ pub use dbg_pls_derive::DebugPls;
 ///
 /// let origin = Point { x: 0, y: 0 };
 ///
-/// assert_eq!(format!("The origin is: {}", debug(&origin)), "The origin is: Point { x: 0, y: 0 }");
+/// assert_eq!(format!("The origin is: {}", pretty(&origin)), "The origin is: Point { x: 0, y: 0 }");
 /// ```
 ///
 /// Manually implementing:
 ///
 /// ```
-/// use dbg_pls::{debug, DebugPls, Formatter};
+/// use dbg_pls::{pretty, DebugPls, Formatter};
 /// struct Point {
 ///     x: i32,
 ///     y: i32,
@@ -132,7 +148,7 @@ pub use dbg_pls_derive::DebugPls;
 ///
 /// let origin = Point { x: 0, y: 0 };
 ///
-/// assert_eq!(format!("The origin is: {}", debug(&origin)), "The origin is: Point { x: 0, y: 0 }");
+/// assert_eq!(format!("The origin is: {}", pretty(&origin)), "The origin is: Point { x: 0, y: 0 }");
 /// ```
 pub trait DebugPls {
     /// Formats the value using the given formatter.
@@ -140,7 +156,7 @@ pub trait DebugPls {
     /// # Examples
     ///
     /// ```
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     ///
     /// struct Position {
     ///     longitude: f32,
@@ -157,7 +173,7 @@ pub trait DebugPls {
     /// }
     ///
     /// let position = Position { longitude: 1.987, latitude: 2.983 };
-    /// assert_eq!(format!("{}", debug(&position)), "(1.987, 2.983)");
+    /// assert_eq!(format!("{}", pretty(&position)), "(1.987, 2.983)");
     /// ```
     fn fmt(&self, f: Formatter<'_>);
 }
@@ -186,7 +202,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     ///
     /// struct Foo {
     ///     bar: i32,
@@ -206,10 +222,11 @@ impl<'a> Formatter<'a> {
     ///     baz: "Hello World".to_string(),
     /// };
     /// assert_eq!(
-    ///     format!("{}", debug(&value)),
+    ///     format!("{}", pretty(&value)),
     ///     "Foo { bar: 10, baz: \"Hello World\" }",
     /// );
     /// ```
+    #[must_use]
     pub fn debug_struct(self, name: &str) -> DebugStruct<'a> {
         DebugStruct::new(self, name)
     }
@@ -220,7 +237,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     ///
     /// struct Foo(i32, String);
     ///
@@ -234,8 +251,9 @@ impl<'a> Formatter<'a> {
     /// }
     ///
     /// let value = Foo(10, "Hello".to_string());
-    /// assert_eq!(format!("{}", debug(&value)), "(10, \"Hello\")");
+    /// assert_eq!(format!("{}", pretty(&value)), "(10, \"Hello\")");
     /// ```
+    #[must_use]
     pub fn debug_tuple(self) -> DebugTuple<'a> {
         DebugTuple::new(self)
     }
@@ -246,7 +264,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     ///
     /// struct Foo(i32, String);
     ///
@@ -260,8 +278,9 @@ impl<'a> Formatter<'a> {
     /// }
     ///
     /// let value = Foo(10, "Hello".to_string());
-    /// assert_eq!(format!("{}", debug(&value)), "Foo(10, \"Hello\")");
+    /// assert_eq!(format!("{}", pretty(&value)), "Foo(10, \"Hello\")");
     /// ```
+    #[must_use]
     pub fn debug_tuple_struct(self, name: &str) -> DebugTupleStruct<'a> {
         DebugTupleStruct::new(self, name)
     }
@@ -272,7 +291,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     ///
     /// struct Foo(Vec<i32>);
     ///
@@ -283,8 +302,9 @@ impl<'a> Formatter<'a> {
     /// }
     ///
     /// let value = Foo(vec![10, 11]);
-    /// assert_eq!(format!("{}", debug(&value)), "[10, 11]");
+    /// assert_eq!(format!("{}", pretty(&value)), "[10, 11]");
     /// ```
+    #[must_use]
     pub fn debug_list(self) -> DebugList<'a> {
         DebugList::new(self)
     }
@@ -295,7 +315,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     /// use std::collections::BTreeMap;
     ///
     /// struct Foo(BTreeMap<String, i32>);
@@ -310,13 +330,14 @@ impl<'a> Formatter<'a> {
     ///     ("World".to_string(), 10),
     /// ]));
     /// assert_eq!(
-    ///     format!("{}", debug(&value)),
+    ///     format!("{}", pretty(&value)),
     /// "{
     ///     [\"Hello\"] = 5;
     ///     [\"World\"] = 10;
     /// }",
     /// );
     /// ```
+    #[must_use]
     pub fn debug_map(self) -> DebugMap<'a> {
         DebugMap::new(self)
     }
@@ -327,7 +348,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     /// use std::collections::BTreeSet;
     ///
     /// struct Foo(BTreeSet<String>);
@@ -342,13 +363,14 @@ impl<'a> Formatter<'a> {
     ///     "World".to_string(),
     /// ]));
     /// assert_eq!(
-    ///     format!("{}", debug(&value)),
+    ///     format!("{}", pretty(&value)),
     /// "{
     ///     \"Hello\";
     ///     \"World\"
     /// }",
     /// );
     /// ```
+    #[must_use]
     pub fn debug_set(self) -> DebugSet<'a> {
         DebugSet::new(self)
     }
@@ -358,7 +380,7 @@ impl<'a> Formatter<'a> {
     /// # Examples
     ///
     /// ```rust
-    /// use dbg_pls::{debug, DebugPls, Formatter};
+    /// use dbg_pls::{pretty, DebugPls, Formatter};
     ///
     /// struct Foo;
     ///
@@ -368,7 +390,7 @@ impl<'a> Formatter<'a> {
     ///     }
     /// }
     ///
-    /// assert_eq!(format!("{}", debug(&Foo)), "Foo");
+    /// assert_eq!(format!("{}", pretty(&Foo)), "Foo");
     /// ```
     pub fn debug_ident(self, name: &str) {
         let path: syn::Path = syn::Ident::new(name, Span::call_site()).into();
@@ -376,7 +398,7 @@ impl<'a> Formatter<'a> {
             attrs: vec![],
             qself: None,
             path,
-        })
+        });
     }
 }
 
@@ -399,7 +421,7 @@ mod tests {
             foo: 5,
             bar: "hello",
         };
-        assert_eq!(debug(&val).to_string(), r#"Demo { foo: 5, bar: "hello" }"#);
+        assert_eq!(pretty(&val).to_string(), r#"Demo { foo: 5, bar: "hello" }"#);
     }
 
     #[test]
@@ -409,7 +431,7 @@ mod tests {
             bar: "Hello, world! I am a very long string",
         };
         assert_eq!(
-            debug(&val).to_string(),
+            pretty(&val).to_string(),
             r#"Demo {
     foo: 5,
     bar: "Hello, world! I am a very long string",
@@ -426,7 +448,7 @@ mod tests {
         val[6].bar = "Hello, world! I am a very long string";
 
         assert_eq!(
-            debug(&val).to_string(),
+            pretty(&val).to_string(),
             r#"[
     Demo { foo: 5, bar: "hello" },
     Demo { foo: 5, bar: "hello" },
@@ -450,7 +472,7 @@ mod tests {
         let set = BTreeSet::from([420, 69]);
 
         assert_eq!(
-            debug(&set).to_string(),
+            pretty(&set).to_string(),
             r#"{
     69;
     420
@@ -472,7 +494,7 @@ mod tests {
         ]);
 
         assert_eq!(
-            debug(&set).to_string(),
+            pretty(&set).to_string(),
             r#"{
     Demo {
         foo: 5,
@@ -488,7 +510,7 @@ mod tests {
         let map = BTreeMap::from([("hello", 60), ("Hello, world! I am a very long string", 12)]);
 
         assert_eq!(
-            debug(&map).to_string(),
+            pretty(&map).to_string(),
             r#"{
     ["Hello, world! I am a very long string"] = 12;
     ["hello"] = 60;
@@ -516,7 +538,7 @@ mod tests {
         ]);
 
         assert_eq!(
-            debug(&map).to_string(),
+            pretty(&map).to_string(),
             r#"{
     [
         Demo {
@@ -528,4 +550,37 @@ mod tests {
 }"#
         );
     }
+
+    #[derive(DebugPls)]
+    #[dbg_pls(crate = "crate")]
+    pub struct Generic<T> {
+        arg: T,
+    }
+
+    #[test]
+    fn debug_generic() {
+        let generic = Generic { arg: "string" };
+        assert_eq!(pretty(&generic).to_string(), r#"Generic { arg: "string" }"#);
+    }
+
+    #[derive(DebugPls)]
+    #[dbg_pls(crate = "crate")]
+    pub struct Generic2<T> {
+        arg: Wrapped<T>,
+    }
+
+    pub struct Wrapped<T>(T);
+    impl<T> DebugPls for Wrapped<T> {
+        fn fmt(&self, f: Formatter<'_>) {
+            f.debug_struct("Wrapped").finish_non_exhaustive();
+        }
+    }
+
+    pub struct NonDebug;
+
+    #[test]
+    fn debug_generic2() {
+        let generic = Generic { arg: Wrapped(NonDebug) };
+        assert_eq!(pretty(&generic).to_string(), r#"Generic { arg: Wrapped {} }"#);
+    }
 }
